@@ -12,6 +12,7 @@
 /*************************************************************************/
 
 #include "api-structures.h"
+#include "dict-common/dialect.h"      // dialect_alloc
 #include "dict-common/dict-affix.h"
 #include "dict-common/dict-api.h"
 #include "dict-common/dict-common.h"
@@ -21,6 +22,7 @@
 #include "dict-common/idiom.h"
 #include "dict-common/regex-morph.h"
 #include "post-process/pp_knowledge.h"
+#include "read-dialect.h"
 #include "read-dict.h"
 #include "read-regex.h"
 #include "string-set.h"
@@ -136,8 +138,15 @@ dictionary_six_str(const char * lang,
 		dict->lookup_wild = file_lookup_wild;
 		dict->free_lookup = free_llist;
 		dict->lookup = file_boolean_lookup;
+		dict->dialect_tag.set = string_id_create();
 		condesc_init(dict, 1<<13);
 		Exp_pool_size = 1<<13;
+
+		if (!test_enabled("no-macro-tag"))
+		{
+			dict->macro_tag = malloc(sizeof(*dict->macro_tag));
+			memset(dict->macro_tag, 0, sizeof(*dict->macro_tag));
+		}
 	}
 	else
 	{
@@ -178,6 +187,12 @@ dictionary_six_str(const char * lang,
 		return dict;
 	}
 
+	if (dict->dialect_tag.num == 0)
+	{
+		string_id_delete(dict->dialect_tag.set);
+		dict->dialect_tag.set = NULL;
+	}
+
 	dictionary_setup_locale(dict);
 
 	dict->affix_table = dictionary_six(lang, affix_name, NULL, NULL, NULL, NULL);
@@ -282,6 +297,28 @@ Dictionary dictionary_create_from_file(const char * lang)
 		free(cons_name);
 		free(pp_name);
 		free(dict_name);
+
+		if (dictionary != NULL)
+		{
+			char *dialect_name = join_path(lang, "4.0.dialect");
+			if (!dialect_file_read(dictionary, dialect_name))
+			{
+				dictionary_delete(dictionary);
+				dictionary = NULL;
+			}
+			else if ((dictionary->dialect == NULL) ||
+			         (dictionary->dialect->num_table_tags == 0))
+			{
+				/* No or empty dialect file. */
+				free_dialect(dictionary->dialect);
+				dictionary->dialect = NULL;
+			}
+			free(dialect_name);
+
+			if (0)
+			{
+			}
+		}
 	}
 	else
 	{
