@@ -25,6 +25,8 @@
 
 /* This file contains the exhaustive search algorithm. */
 
+#define D_COUNT 5 /* General debug level for this file. */
+
 typedef struct Table_connector_s Table_connector;
 struct Table_connector_s
 {
@@ -57,6 +59,7 @@ struct count_context_s
 	/* int     null_block; */ /* not used, always 1 */
 	bool    islands_ok;
 	bool    exhausted;
+	bool    keep_table;
 	unsigned int checktimer;  /* Avoid excess system calls */
 	unsigned int table_size;
 	unsigned int table_lrcnt_size;
@@ -69,13 +72,26 @@ struct count_context_s
 
 static void free_table(count_context_t *ctxt)
 {
-	xfree(ctxt->table, ctxt->table_size * sizeof(Table_connector*));
+	if (!ctxt->keep_table) free(ctxt->table);
 	ctxt->table = NULL;
 	ctxt->table_size = 0;
 }
 
+static void init_table(count_context_t *ctxt, size_t sent_len);
+static void free_kept_table(void)
+{
+	init_table(NULL, 0);
+}
+
 static void init_table(count_context_t *ctxt, size_t sent_len)
 {
+	static TLS Table_connector **kept_table;
+
+	if (ctxt == NULL)
+	{
+		free(kept_table);
+		return;
+	}
 	if (ctxt->table) free_table(ctxt);
 
 	/* A piecewise exponential function determines the size of the
@@ -85,33 +101,73 @@ static void init_table(count_context_t *ctxt, size_t sent_len)
 	unsigned int shift;
 	if (sent_len >= 10)
 	{
-		shift = 12 + (sent_len) / 4 ;
+		shift = 12 + (sent_len) / 4;
 	}
 	else
 	{
 		shift = 12;
 	}
 
-	/* Clamp at max 4*(1<<24) == 64 MBytes */
-	if (24 < shift) shift = 24;
-	lgdebug(+5, "Connector table size (1<<%u)*%zu\n", shift, sizeof(Table_connector));
+#define MAX_LOG2_TABLE_SIZE 24
+	/* Clamp at max 8*(1<<MAX_LOG2_TABLE_SIZE)==128 MBytes on 64 bit systems. */
+	if (MAX_LOG2_TABLE_SIZE < shift) shift = MAX_LOG2_TABLE_SIZE;
+	lgdebug(+D_COUNT, "Connector table size (1<<%u)*%zu\n", shift, sizeof(Table_connector));
 	ctxt->table_size = (1U << shift);
 	/* ctxt->log2_table_size = shift; */
-	ctxt->table = (Table_connector**)
-		xalloc(ctxt->table_size * sizeof(Table_connector*));
+	ctxt->table = (kept_table != NULL) ? kept_table :
+		malloc(ctxt->table_size * sizeof(Table_connector*));
 	memset(ctxt->table, 0, ctxt->table_size*sizeof(Table_connector*));
+
+	if (kept_table != NULL)
+	{
+		ctxt->keep_table = true;
+	}
+	else
+	{
+		if (MAX_LOG2_TABLE_SIZE == shift)
+		{
+			/* The maximum size table has just been allocated. Arrange for
+			 * keeping it allocated until existing, so it can be reused. This
+			 * avoids a big overhead of malloc/free of large memory blocks on
+			 * systems that use mmap/munmap for that (like Linux). */
+			kept_table = ctxt->table;
+			ctxt->keep_table = true;
+			atexit(free_kept_table);
+		}
+	}
 }
 
 static void free_table_lrcnt(count_context_t *ctxt)
 {
-	if (verbosity_level(+5))
+	if (verbosity_level(D_COUNT))
 	{
 		unsigned int table_usage = MAX_TABLE_LRCNT_SIZE(ctxt->table_lrcnt_size) -
 			ctxt->table_lrcnt_available_count;
+		unsigned int nonzero = 0;
+		unsigned int any_null = 0;
+		unsigned int zero = 0;
+		unsigned int non_max_null = 0;
 
-		lgdebug(+0, "free_table_lrcnt: Usage %u/%u %.2f%%\n",
+		for (unsigned int i = 0; i < ctxt->table_lrcnt_size; i++)
+		{
+			if (ctxt->table_lrcnt[i].tracon_id == -1) continue;
+
+			if (ctxt->table_lrcnt[i].status == 1)
+				nonzero++;
+			else if (ctxt->table_lrcnt[i].null_count == ANY_NULL_COUNT)
+				any_null++;
+			else if (ctxt->sent->null_count > ctxt->table_lrcnt[i].null_count)
+				non_max_null++;
+			else if (ctxt->sent->null_count == ctxt->table_lrcnt[i].null_count)
+				zero++;
+		}
+
+		lgdebug(+0, "Usage %u/%u %.2f%% "
+		        "(usage = non_max_null %u + other %u, "
+		        "other = any_null_zero %u + zero %u + nonzero %u)\n",
 		        table_usage, ctxt->table_lrcnt_size,
-		        100.0f*table_usage / ctxt->table_lrcnt_size);
+		        100.0f*table_usage / ctxt->table_lrcnt_size,
+		        non_max_null, table_usage-non_max_null, any_null, zero, nonzero);
 	}
 
 	free(ctxt->table_lrcnt);
@@ -124,7 +180,7 @@ static void init_table_lrcnt(count_context_t *ctxt, Sentence sent)
 	if (ctxt->table_lrcnt == NULL)
 	{
 		ctxt->table_lrcnt_size =
-			next_power_of_two_up(2048 + sent->num_disjuncts * sent->length / 16);
+			next_power_of_two_up(2048 + sent->num_disjuncts * sent->length / 1.5f);
 		ctxt->table_lrcnt = malloc(ctxt->table_lrcnt_size * sizeof(Table_lrcnt));
 	}
 
@@ -222,7 +278,7 @@ static void table_stat(count_context_t *ctxt, Sentence sent)
 	int null_count[256] = { 0 };   /* null_count histogram */
 	int chain_length[64] = { 0 };  /* Chain length histogram */
 	bool table_stat_entries = test_enabled("count-table-entries");
-	int n;              /* For printf() pretty printing. */
+	int n;              /* For printf() pretty printing */
 
 	for (unsigned int i = 0; i < ctxt->table_size; i++)
 	{
@@ -238,7 +294,7 @@ static void table_stat(count_context_t *ctxt, Sentence sent)
 				nz++;
 			null_count[t->null_count]++;
 		}
-		if (c > 0) /* Slot 0 used for length overflow. */
+		if (c > 0) /* Slot 0 used for length overflow */
 		{
 			chain_length[c >= (int)ARRAY_SIZE(chain_length) ? 0 : c]++;
 			total_c += c;
@@ -253,7 +309,7 @@ static void table_stat(count_context_t *ctxt, Sentence sent)
 #endif
 	int used_slots = ctxt->table_size-N;
 	/* The used= value is TotalValues/TableSize (not UsedSlots/TableSize). */
-	printf("Connector table: msb=%d slots=%6d/%6d (%5.2f%%) avg-chain=%4.2f "
+	printf("Connector table: msb=%d slots=%6d/%6u (%5.2f%%) avg-chain=%4.2f "
 	       "values=%6d (z=%5d nz=%5d N=%5d) used=%5.2f%% "
 	       "acc=%zu (hit=%zu miss=%zu) (sent_len=%zu)\n",
 	       msb(ctxt->table_size), used_slots, ctxt->table_size,
@@ -272,7 +328,7 @@ static void table_stat(count_context_t *ctxt, Sentence sent)
 		for (unsigned int nc = 0; nc < ARRAY_SIZE(null_count); nc++)
 		{
 			if (0 != null_count[nc])
-				printf("%d: %d\n", nc, null_count[nc]);
+				printf("%u: %d\n", nc, null_count[nc]);
 		}
 	}
 
@@ -282,14 +338,14 @@ static void table_stat(count_context_t *ctxt, Sentence sent)
 		{
 			if (0 == null_count[nc]) continue;
 
-			printf("Null count %d:\n", nc);
+			printf("Null count %u:\n", nc);
 			for (unsigned int i = 0; i < ctxt->table_size; i++)
 			{
 				for (Table_connector *t = ctxt->table[i]; t != NULL; t = t->next)
 				{
 					if (t->null_count != nc) continue;
 
-					printf("[%d]%n", i, &n);
+					printf("[%u]%n", i, &n);
 					printf("%*d %5d c=%lld\n",  15-n, t->l_id, t->r_id, t->count);
 				}
 			}
@@ -300,7 +356,7 @@ static void table_stat(count_context_t *ctxt, Sentence sent)
 }
 #else
 #define DEBUG_TABLE_STAT(x)
-#endif /* DEBUG */
+#endif /* DEBUG  */
 
 /**
  * Stores the value in the table.  Assumes it's not already there.
@@ -332,7 +388,6 @@ find_table_pointer(count_context_t *ctxt,
                    const Connector *le, const Connector *re,
                    unsigned int null_count)
 {
-
 	int l_id = (NULL != le) ? le->tracon_id : lw;
 	int r_id = (NULL != re) ? re->tracon_id : rw;
 
@@ -367,7 +422,7 @@ Count_bin* table_lookup(count_context_t * ctxt,
  * Cache lookup:
  * Is the range [c, w) going to yield a nonzero leftcount / rightcount?
  *
- * @param ctxt Count context
+ * @param ctxt Count context.
  * @param dir Direction - 0: leftcount, 1: rightcount.
  * @param c The connector that starts the range.
  * @w The word that ends the range.
@@ -380,7 +435,7 @@ Count_bin* table_lookup(count_context_t * ctxt,
  * @return Cache entry for the given range. Possible values:
  *    NULL - A nonzero count may be encountered for null_count>=lnull_start.
  *    lrcnt_cache_zero - A zero count would result.
- *    Cache pointer - an update for null_count>=lnull_start is needed.
+ *    Cache pointer - An update for null_count>=lnull_start is needed.
  */
 static Table_lrcnt *is_lrcnt(count_context_t *ctxt, int dir,
                               Connector *c, int cw, int w,
@@ -420,7 +475,7 @@ static Table_lrcnt *is_lrcnt(count_context_t *ctxt, int dir,
 		 * been handled yet for the given range (the cache will be
 		 * updated for them by lrcnt_cache_update()). */
 		if (null_start == NULL) return NULL;
-		*null_start = (null_count_m)(lrcnt_cache->null_count + 1);
+		*null_start = lrcnt_cache->null_count + 1;
 	}
 
 	return lrcnt_cache;
@@ -532,7 +587,7 @@ static Count_bin do_count(int lineno, count_context_t *ctxt,
 		snprintf(m_result, sizeof(m_result), "(M=%lld)", hist_total(&t->count));
 
 	level++;
-	prt_error("%*sdo_count%s:%d lw=%d rw=%d le=%s(%d) re=%s(%d) null_count=%d\n\\",
+	prt_error("%*sdo_count%s:%d lw=%d rw=%d le=%s(%d) re=%s(%d) null_count=%u\n\\",
 		level*2, "", m_result, lineno, lw, rw, V(le),ID(le,lw), V(re),ID(re,rw), null_count);
 	Count_bin r = do_count1(lineno, ctxt, lw, rw, le, re, null_count);
 	prt_error("%*sreturn%.*s:%d=%lld\n",
@@ -559,7 +614,7 @@ static Count_bin do_count(
 	Table_connector *t;
 
 	/* TODO: static_assert() that null_count is an unsigned int. */
-	assert (null_count < INT_MAX, "Bad null count %d", null_count);
+	assert (null_count < INT_MAX, "Bad null count %d", (int)null_count);
 
 	t = find_table_pointer(ctxt, lw, rw, le, re, null_count);
 
@@ -628,7 +683,7 @@ static Count_bin do_count(
 		 * of null words. */
 		t->count = zero;
 		w = lw + 1;
-		for (int opt = 0; opt <= !!ctxt->sent->word[w].optional; opt++)
+		for (int opt = 0; opt <= (int)ctxt->sent->word[w].optional; opt++)
 		{
 			null_count += opt;
 
@@ -659,22 +714,51 @@ static Count_bin do_count(
 		return t->count;
 	}
 
+	/* The word range (lw, rw) gets split in all tentatively possible ways
+	 * to LHS term and RHS term.
+	 * There can be a total count > 0 only if one of the following
+	 * multiplications results in a value > 0. This in turn is possible
+	 * only if both multiplication terms > 0:
+	 *    LHS term    RHS term
+	 * 1. leftcount x rightcount
+	 * 2. leftcount x l_bnr
+	 * 3. r_bnl     x rightcount
+	 *
+	 * In addition:
+	 * - leftcount > 0   is possible only if there is match_left.
+	 * - rightcount > 0  is possible only if there is match_right.
+	 * - r_bnl > 0       is possible only if le==NULL.
+	 *
+	 * Since the result count is a sum of multiplications of the LHS and
+	 * RHS counts, if one of them is zero, we can skip calculating the
+	 * other one.
+	 */
+
 	if (le == NULL)
 	{
-		start_word = lw+1;
+		/* No leftcount, and no rightcount for (w < re->farthest_word). */
+		start_word = MAX(lw+1, re->farthest_word);
 	}
 	else
 	{
+		/* The following cannot be optimized like end_word due to l_bnr. */
 		start_word = le->nearest_word;
 	}
 
 	if (re == NULL)
 	{
-		end_word = rw;
+		/* No rightcount, and no leftcount for (w > le->farthest_word). */
+		end_word = MIN(rw, le->farthest_word+1);
 	}
 	else
 	{
-		end_word = re->nearest_word +1;
+		/* If the LHS count for a word would be zero for a left-end connector
+		 * due to the distance of this word, we can skip its handling
+		 * entirely. So the checked word interval can be shortened. */
+		if ((le != NULL) && (re->nearest_word > le->farthest_word))
+			end_word = le->farthest_word + 1;
+		else
+			end_word = re->nearest_word + 1;
 	}
 
 	total = zero;
@@ -721,8 +805,9 @@ static Count_bin do_count(
 				if (rgc == &lrcnt_cache_zero) fml_re = NULL;
 			}
 		}
-		else if (re != NULL)
+		else
 		{
+			/* Here re != NULL. */
 			unsigned int rnull_start;
 			lrcnt_cache = is_lrcnt(ctxt, 1, re, rw, w, null_count, &rnull_start);
 			if (lrcnt_cache == &lrcnt_cache_zero) continue;
@@ -731,7 +816,7 @@ static Count_bin do_count(
 			{
 				lrcnt_optimize = false;
 				if (rnull_start <= null_count)
-					lnull_end = null_count - rnull_start;
+					lnull_end -= rnull_start;
 			}
 		}
 		/* End of nonzero leftcount/rightcount range cache check. */
@@ -750,7 +835,7 @@ static Count_bin do_count(
 			bool Rmatch = d->match_right;
 
 #ifdef VERIFY_MATCH_LIST
-			assert(id == d->match_id, "Modified id (%d!=%d)", id, d->match_id);
+			assert(id == d->match_id, "Modified id (%u!=%u)", id, d->match_id);
 #endif
 
 			for (unsigned int lnull_cnt = lnull_start; lnull_cnt <= lnull_end; lnull_cnt++)
@@ -770,7 +855,7 @@ static Count_bin do_count(
 				Count_bin r_cmulti = INIT_NO_COUNT;
 				Count_bin r_dmulti = INIT_NO_COUNT;
 				Count_bin r_dcmulti = INIT_NO_COUNT;
-				Count_bin r_bnl = INIT_NO_COUNT;
+				Count_bin r_bnl = (le == NULL) ? INIT_NO_COUNT : 0;
 
 				/* Now, we determine if (based on table only) we can see that
 				   the current range is not parsable. */
@@ -971,10 +1056,8 @@ static Count_bin do_count(
  * used anywhere, and a 3-5% speedup is available if it is avoided.
  * We plan to use this histogram, later ....
  */
-Count_bin do_parse(Sentence sent,
-                   fast_matcher_t *mchxt,
-                   count_context_t *ctxt,
-                   int null_count, Parse_Options opts)
+int do_parse(Sentence sent, fast_matcher_t *mchxt, count_context_t *ctxt,
+             Parse_Options opts)
 {
 	Count_bin hist;
 
@@ -987,9 +1070,9 @@ Count_bin do_parse(Sentence sent,
 	/* Cannot reuse since its content is invalid on an increased null_count. */
 	init_table_lrcnt(ctxt, sent);
 
-	hist = do_count(ctxt, -1, sent->length, NULL, NULL, null_count+1);
+	hist = do_count(ctxt, -1, sent->length, NULL, NULL, sent->null_count+1);
 
-	DEBUG_TABLE_STAT(if (verbosity_level(+5)) table_stat(ctxt, sent));
+	DEBUG_TABLE_STAT(if (verbosity_level(+D_COUNT)) table_stat(ctxt, sent));
 
 	return hist;
 }
