@@ -561,6 +561,11 @@ const char * dyn_str_value(dyn_str* s)
 	return s->str;
 }
 
+size_t dyn_strlen(dyn_str* s)
+{
+	return s->end;
+}
+
 /* ======================================================== */
 /* Locale routines */
 
@@ -747,7 +752,8 @@ static void free_C_LC_NUMERIC(void)
 }
 #endif /* HAVE_LOCALE_T */
 
-bool strtodC(const char *s, double *r)
+/* FIXME: Rewrite to directly convert scaled integer strings (only). */
+bool strtodC(const char *s, float *r)
 {
 	char *err;
 
